// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "UIMesh.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

UIMesh::UIMesh(SceneItem *i) :
  texture(nullptr),
  UIComponentRenderable(i)
{

}

float_t UIMesh::getContentWidth() {
  if(this->texture != nullptr) return this->texture->getWidth();
  return this->width;
}

float_t UIMesh::getContentHeight() {
  if(this->texture != nullptr) return this->texture->getHeight();
  return this->height;
}

std::vector<struct ShaderPassItem> UIMesh::getUIRenderPasses() {
  struct ShaderPassItem item;
  auto shader = getGame()->renderManager.uiShader;
  item.shader = shader;
  item.colorValues[shader->paramColor] = this->color;
  item.parameterBuffers[shader->bufferUiCanvas] = &getCanvas()->shaderBuffer;
  item.matrixValues[shader->paramModel] = this->transform->getWorldTransform();
  if(this->texture == nullptr) {
    item.boolValues[shader->paramHasTexture] = false;
  } else {
    item.boolValues[shader->paramHasTexture] = true;
    item.textureSlots[0] = this->texture;
    item.textureValues[shader->paramTexture] = 0;
  }
  item.w = this->transform->getWorldPosition().z;
  item.renderFlags = RENDER_MANAGER_RENDER_FLAG_BLEND;
  item.mesh = &mesh;
  return { item };
}

void UIMesh::onStart() {
  useEffect([&]{

  }, this->positions)();

  useEffect([&]{

  }, this->uvs)();
}